local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicMain.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
-- Apply building asset processor
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofMain.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
-- Apply building asset processor
StoneGate:registerAssetProcessor("models/roofs/domeRoofs/domeMain.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
-- Apply building asset processor
StoneGate:registerAssetProcessor("models/ramparts/rampartMain.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register main materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofMain.fbx/Materials/redRoof", StoneGateSetPrefix .. "_MAIN_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/roofs/domeRoofs/domeMain.fbx/Materials/redRoof", StoneGateSetPrefix .. "_MAIN_DOME_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicMain.fbx/Materials/murChateau", StoneGateSetPrefix .. "_MAIN_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartMain.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_MAIN_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicMain.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_MAIN_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicMain.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_MAIN_WHITE_STONE_FLOOR_GROUND_MATERIAL")

-- Register Main prefab nodes
StoneGate:registerAssetId("models/basic/basicMain.fbx/Prefab/main/step_3_Main_TopPart", StoneGateSetPrefix .. "_MAIN_TOP_PREFAB")

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:registerAsset({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = true, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }}
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

registerDefaultBuildingPart(StoneGateSetPrefix .. "_MAIN_TOP")

function registerMainBuilding(_option)
    -- Register Main prefab nodes
    StoneGate:registerAssetId("models/basic/basicMain.fbx/Prefab/main/step_0_Main_Root" .. _option .. "Part", StoneGateSetPrefix .. "_MAIN_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicMain.fbx/Prefab/main/step_1_Main_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_MAIN_TILING1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicMain.fbx/Prefab/main/step_2_Main_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_MAIN_TILING2" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_MAIN_TILING1" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_MAIN_TILING2" .. _option)

    -- Register the stone gate set main root part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_MAIN_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_MAIN_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_MAIN_ROOT" .. _option .. "_DESC",
	Category = "CORE",
	BuildingZone = _MainBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_MAIN_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_MAIN_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_MAIN_TILING2" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_MAIN_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 12,
	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "MAIN_ROOT" .. _option)
    })

    StoneGate:registerPrefabComponent("models/basic/basicMain.fbx/Prefab/main/step_0_Main_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 8, 12 }
    })
end

registerMainBuilding("") -- with stone ground
registerMainBuilding("P") -- with wooden ground
registerMainBuilding("HALFWHITE") -- Half white with stone ground

-- Roofs...

function registerRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _angle)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. "/Prefab/main/Main_" .. _nameFbx .. _angle .. "Part", StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. _angle .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. _angle .. "_PART",
	Name = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. _angle .. "_NAME",
	Description = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. _angle .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. _angle .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5*_NoSandbox }
		}}
	}
    })
end

registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof2Opposite", "ROOF_2OPPOSITE", "45" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof1Opened",   "ROOF_1OPENED",   "45" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof4Closed",   "ROOF_4CLOSED",   "45" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof3Shed",     "ROOF_3SHED",     "45" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof2Opposite", "ROOF_2OPPOSITE", "30" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof1Opened",   "ROOF_1OPENED",   "30" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof4Closed",   "ROOF_4CLOSED",   "30" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof3Shed",     "ROOF_3SHED",     "30" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof3Shed",     "ROOF_3SHED",     "24" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof2Opposite", "ROOF_2OPPOSITE", "60" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof1Opened",   "ROOF_1OPENED",   "60" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof4Closed",   "ROOF_4CLOSED",   "60" )
registerRoofPart( "roofs/simpleRoofs/roofMain.fbx", "Roof3Shed",     "ROOF_3SHED",     "60" )
registerRoofPart( "roofs/domeRoofs/domeMain.fbx", "RoofDome2Opposite", "ROOF_DOME_2OPPOSITE", "" )
registerRoofPart( "roofs/domeRoofs/domeMain.fbx", "RoofDome1Opened",   "ROOF_DOME_1OPENED",   "" )
registerRoofPart( "roofs/domeRoofs/domeMain.fbx", "RoofDome4Closed",   "ROOF_DOME_4CLOSED",   "" )

---- Ramparts...

function registerRampartPart(_nameFbxFile, _nameFbx, _nameAsset)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. ".fbx/Prefab/main/step_4_Main_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_MAIN_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	}
    })
end

registerRampartPart( "ramparts/rampartMain", "Rampart2Opposite", "RAMPART_2OPPOSITE" )
registerRampartPart( "ramparts/rampartMain", "Rampart1Opened",   "RAMPART_1OPENED" )
registerRampartPart( "ramparts/rampartMain", "Rampart4Closed",   "RAMPART_4CLOSED" )
